extern crate lapp;

extern crate rustfft;
use rustfft::FFTplanner;
use rustfft::num_complex::Complex;
use rustfft::num_traits::Zero;

use std::io::prelude::*;
use std::io::stdout;
use std::io::ErrorKind;

fn main() {
	let args = lapp::parse_args("
		termfall terminal waterfall plotter v0.1
		-i, --input (default stdin) file from which to read samples
		-s, --size (default 1024) FFT size to use
		-z, --zoom (default 1) integer factor by which to zoom in on the lower frequencies
		-d, --decimation (default 1) decimation factor
		-a, --autoscale whether or not to automatically vary sensitivity with input
		-m, --max (default 1000.0) scaling factor for plot sensitivity, smaller is more sensitive
		-w, --width (default 120) width of the plot in characters
	");
	let mut source = args.get_infile("input");
	let fft_size = args.get_integer("size") as usize;
	let fft_window_size = fft_size/(2*args.get_integer("zoom") as usize);
	let decimation = args.get_integer("decimation") as usize;
	let autoscale = args.get_bool("autoscale");
	let startmax = args.get_float("max");
	let width = args.get_integer("width") as usize;

	let mut ibuffer:Vec<u8> = vec![0x00; fft_size];

	let mut fft_i:Vec<Complex<f32>> = vec![Zero::zero(); fft_size];
	let mut fft_o:Vec<Complex<f32>> = vec![Zero::zero(); fft_size];
	let mut planner = FFTplanner::new(false);
	let fft = planner.plan_fft(fft_size);

	let mut cyclecount:usize = 0;
	let mut scalemax:f32 = startmax;

	let colors:Vec<u8> = vec![16,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255];

	let sout = stdout();
	let mut slock = sout.lock();

	loop {
		match source.read_exact(&mut ibuffer) {
			Err(why) => match why.kind() {
				ErrorKind::UnexpectedEof => break,
				_ => {
					eprintln!("read error: {}", why);
					return;
				}
			},
			Ok(_) => (),
		};
		cyclecount = cyclecount.wrapping_add(1);
		if cyclecount % decimation != 0 {
			continue;
		}
		for i in 0..fft_size {
			fft_i[i] = Complex::new((ibuffer[i] as f32)-128.0, 0.0f32);
		}
		fft.process(&mut fft_i, &mut fft_o);
		let fft_window = fft_o[..fft_window_size].to_vec();
		let mut fft_lo = Vec::with_capacity(fft_window_size);
		let mut filter:f32 = 0.0;
		for i in 0..fft_window_size {
			filter = (filter*(fft_window_size as f32)*20.0 + fft_window[i].norm_sqr()*(width as f32)*80.0)/(((fft_window_size+width) as f32)*100.0);
			fft_lo.push(filter);
		}
		let mut linemax:f32 = 0.0;
		let mut linestring:String = String::new();
		for i in 0..width {
			let fft_pos_1 = (2*i+0)*fft_window_size/(width*2);
			let fft_pos_2 = (2*i+1)*fft_window_size/(width*2);
			let power1 = *fft_lo.get(fft_pos_1).unwrap_or(&0.0);
			let power2 = *fft_lo.get(fft_pos_2).unwrap_or(&0.0);
			if power1 > linemax {
				linemax = power1;
			}
			if power2 > linemax {
				linemax = power2;
			}
			let mut colorindex1 = ((power1/scalemax)*colors.len() as f32).round() as usize;
			let mut colorindex2 = ((power2/scalemax)*colors.len() as f32).round() as usize;
			if colorindex1 >= colors.len() {
				colorindex1 = colors.len()-1;
			}
			if colorindex2 >= colors.len() {
				colorindex2 = colors.len()-1;
			}
			linestring.push_str(&format!("\x1b[38;5;{}m\x1b[48;5;{}m▌", colors[colorindex1], colors[colorindex2]));
			//print!("{:10}", power.round());
		}
		linestring.push_str("\x1b[0m\n");
		let _ = slock.write_all(&linestring.as_bytes());
		let _ = slock.flush();
		if autoscale {
			scalemax = scalemax*0.99 + linemax*0.01;
		}
	}
}
